#
#  Name:    MpCa.py
#
#  Purpose: MX Python interface to EPICS Channel Access.
#
#  Author:  William Lavender
#
#---------------------------------------------------------------------------
#
#  Copyright 2009, 2011 Illinois Institute of Technology
#
#  See the file "LICENSE" for information on usage and redistribution
#  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

import time

import Mp
import MpCaPrivate

DBE_VALUE = 1
DBE_ALARM = 2
DBE_LOG   = 4

MpCaPrivate.init()

#---------------------------------------------------------------------------

def pend_io( timeout ):
	MpCaPrivate.pend_event( timeout )

def pend_event( timeout ):
	MpCaPrivate.pend_event( timeout )

def poll():
	MpCaPrivate.pend_event( 1.0e-12 )

def flush_io():
	MpCaPrivate.flush_io()

#---------------------------------------------------------------------------

class Callback:
	def __init__( self, callback_mask, \
			callback_function, callback_arguments ):
		self.debug = False

		if self.debug:
			print "MpCa.Callback( %s, %s, %s ) invoked" \
		    % ( callback_mask, callback_function, callback_arguments )

		self.callback_mask = callback_mask
		self.callback_function = callback_function
		self.callback_arguments = callback_arguments

		if self.debug:
			print "MpCa.Callback() constructor complete for %s" \
				% self

	def __del__( self ):
		if self.debug:
			print "MpCa.Callback() destructor invoked for %s" % self
			
		MpCaPrivate.cb_delete_callback( self )

		if self.debug:
			print "MpCa.Callback() destructor complete for %s" \
				% self

#---------------------------------------------------------------------------

class PV:
	def __init__( self, pvname ):
		self.debug = False

		if self.debug:
			print "MpCa.PV( %s ) invoked" % pvname

		self.pvname = pvname

		self.handle = MpCaPrivate.pv_init_stage1( pvname )

		#---
		# We loop in Python waiting for the PV to connect
		# instead of looping in C so that the C-based code
		# used to call Python-based EPICS callbacks gets
		# a chance to grab the Global Interpreter Lock
		# while waiting for the connection.
		#

		timeout = 10.0	;# in seconds

		current_time = time.time()

		timeout_time = current_time + timeout;

		while True:
			poll()

			current_time = time.time()

			connected = self.is_connected()

			if ( connected ):
				break

			if ( current_time >= timeout_time ):
				error_string = \
		"Timed out after waiting %f seconds for PV '%s' to connect." \
					% ( timeout, pvname )

				raise Mp.Timed_Out_Error, error_string

			time.sleep(0.01)

		#---

		MpCaPrivate.pv_init_stage2( self.handle )

	def is_connected( self ):
		status = MpCaPrivate.pv_is_connected( self.handle )

		return status

	def caget( self, timeout=-1.0, remote=True ):
		result = MpCaPrivate.pv_caget( self.handle, timeout, remote )

		if self.debug:
			print "MpCa.PV.caget() received %s from '%s'" \
				% (str(result), self.pvname)

		return result

	def caput( self, args, timeout=-1.0, wait=True ):
		if self.debug:
			print "MpCa.PV.caput() sending %s to '%s'" \
				% (str(args), self.pvname)

		MpCaPrivate.pv_caput( self.handle, timeout, wait, args )

	def get_local( self ):
		result = MpCaPrivate.pv_caget( self.handle, -1.0, False )

		if self.debug:
			print "MpCa.PV.get_local() has %s for '%s'" \
				% (str(result), self.pvname)

		return result

	def add_callback( self, \
			callback_mask, \
			callback_function, \
			callback_arguments ):

		if self.debug:
			print "MpCa.PV.add_callback() for PV '%s'" % self.pvname
			print \
	"  callback_mask = %s, callback_function = %s, callback_arguments = %s"\
		    % ( callback_mask, callback_function, callback_arguments )

		callback = Callback( callback_mask, \
				callback_function, callback_arguments )

		callback.pv = self

		MpCaPrivate.pv_add_callback( self.handle, callback )

		if self.debug:
			print "  callback =", callback

		return callback

